/*
FRODO: a FRamework for Open/Distributed Optimization
Copyright (C) 2008, 2009, 2010, 2011  Thomas Leaute, Brammert Ottens & Radoslaw Szymanek

FRODO is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

FRODO is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.


How to contact the authors: 
<http://lia.epfl.ch/>

EPFL / IC / IIF / LIA
Batiment IN 
Station 14 
CH - 1015 Lausanne 
Switzerland
*/

package ch.epfl.lia.frodo.algorithms.dpop.stochastic;

import java.util.ArrayList;
import java.util.List;

import org.jdom.Document;
import org.jdom.Element;

import ch.epfl.lia.frodo.algorithms.StatsReporter;
import ch.epfl.lia.frodo.solutionSpaces.Addable;

/** The solver for Comp-E[DPOP]
 * @author Thomas Leaute
 * @param <V> type used for variable values
 */
public class Complete_E_DPOPsolver < V extends Addable<V> > extends E_DPOPsolver<V> {
	
	/** The UTIL propagation module */
	private CompleteUTIL<V> utilModule;

	/** Constructor 
	 * @param agentDesc 	description of the agent to be used
	 * @param useTCP 		Whether to use TCP pipes or shared memory pipes
	 * @warning Using TCP pipes automatically disables simulated time. 
	 */
	public Complete_E_DPOPsolver(Document agentDesc, boolean useTCP) {
		super(agentDesc, useTCP);
	}

	/** Constructor 
	 * @param agentDesc 	description of the agent to be used
	 * @param domClass 		the class to use for variable values
	 * @param useTCP 		Whether to use TCP pipes or shared memory pipes
	 * @warning Using TCP pipes automatically disables simulated time. 
	 */
	public Complete_E_DPOPsolver(Document agentDesc, Class<V> domClass, boolean useTCP) {
		super(agentDesc, domClass, useTCP);
	}

	/** Constructor 
	 * @param agentDesc 	description of the agent to be used
	 * @param domClass 		the class to use for variable values
	 */
	public Complete_E_DPOPsolver(Document agentDesc, Class<V> domClass) {
		super(agentDesc, domClass);
	}

	/** Constructor 
	 * @param agentDesc 	description of the agent to be used
	 */
	public Complete_E_DPOPsolver(Document agentDesc) {
		super(agentDesc);
	}

	/** @see E_DPOPsolver#getSolGatherers() */
	@SuppressWarnings("unchecked")
	@Override
	protected ArrayList<StatsReporter> getSolGatherers() {

		ArrayList<StatsReporter> solGatherers = new ArrayList<StatsReporter> (2);

		for (Element elmt : (List<Element>) agentDesc.getRootElement().getChild("modules").getChildren()) {
			String className = elmt.getAttributeValue("className");
			
			if (className.equals(CompleteUTIL.class.getName())) {
				utilModule = new CompleteUTIL<V> (elmt, problem);
				utilModule.setSilent(true);
				solGatherers.add(utilModule);
							
			} else if (className.endsWith("AtLCAs")) { // to display the DFS
//				elmt.setAttribute("DOTrenderer", DOTrenderer.class.getName());
				samplingModule = new SamplingPhase<V> (elmt, problem);
				samplingModule.setSilent(true); // comment this to display the DFS
				solGatherers.add(samplingModule);
			}
		}
		
		return solGatherers;
	}

	/** @see E_DPOPsolver#buildSolution() */
	@Override
	protected StochSolution<V> buildSolution() {
		
		this.dfsString = (this.samplingModule == null ? "" : this.samplingModule.dfsToString());
		
		return new StochSolution<V> (problem.getNbrVars(), utilModule.getOptUtil(), utilModule.getExpectedUtil(), utilModule.getWorstUtil(), utilModule.getProbOfOptimality(), utilModule.getCentralization(), 
				utilModule.getSolution(), factory.getNbrMsgs(), factory.getMsgNbrs(), factory.getTotalMsgSize(), factory.getMsgSizes(), factory.getNcccs(), factory.getTime(), null, utilModule.getMaxMsgDim());
	}
	
	/** @see E_DPOPsolver#clear() */
	@Override
	public void clear () {
		super.clear();
		this.utilModule = null;
	}
	
}
